﻿using System.Collections.ObjectModel;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.Wpf;

namespace GettingStartedSample
{
    public partial class MainWindow : Window
    {
        private LayerOverlay layerOverlay;
        private InMemoryFeatureLayer highlightLayer;
        private string appDataPath = "../../AppData";
        private PointShape worldLocation = new PointShape(0, 0);
        private Collection<Feature> selectedFeatures = new Collection<Feature>();
        InMemoryFeatureLayer AVInMemoryFeatureLayerLoadEnc = new InMemoryFeatureLayer();
        LayerOverlay AVLayerOverlayLoadEnc = new LayerOverlay();

        public MainWindow()
        {
            InitializeComponent();
        }

        private void map_Loaded(object sender, RoutedEventArgs e)
        {
            map.MapUnit = GeographyUnit.DecimalDegree;
            map.CurrentExtent = new RectangleShape(-133.2515625, 89.2484375, 126.9046875, -88.290625);
            map.MapClick += MapClick;

            map.TrackOverlay.TrackShapeLayer.WrappingMode = WrappingMode.WrapDateline;
            map.TrackOverlay.TrackShapeLayer.WrappingExtent = new RectangleShape(-180, 90, 180, -90);
            map.EditOverlay.EditShapesLayer.WrappingMode = WrappingMode.WrapDateline;
            map.EditOverlay.EditShapesLayer.WrappingExtent = new RectangleShape(-180, 90, 180, -90);



            //WorldStreetsAndImageryOverlay worldOverlay = new WorldStreetsAndImageryOverlay();
            //worldOverlay.WrappingMode = WrappingMode.WrapDateline;
            //map.Overlays.Add(worldOverlay);



            //CustomDefaultValueStyle customDefaultValueStyle_Valid = new CustomDefaultValueStyle();
            //customDefaultValueStyle_Valid.ColumnName = "Polygon";
            //customDefaultValueStyle_Valid.DefaultStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(0, 0, 0, 0), GeoColor.FromArgb(500, 255, 0, 0), 1, LineDashStyle.Solid);
            //customDefaultValueStyle_Valid.ValueItems.Add(new ValueItem("Valid", AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(0, 0, 0, 0), GeoColor.FromArgb(500, 0, 204, 0), 1, LineDashStyle.Solid)));
            //customDefaultValueStyle_Valid.ValueItems.Add(new ValueItem("New", AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(0, 0, 0, 0), GeoColor.FromArgb(500, 255, 0, 0), 1, LineDashStyle.Solid)));
            //customDefaultValueStyle_Valid.ValueItems.Add(new ValueItem("AboutToExpire", AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(0, 0, 0, 0), GeoColor.FromArgb(500, 255, 255, 0), 1, LineDashStyle.Solid)));

            //AVCSInMemoryFeatureLayerLoadEnc.Open();
            //AVCSInMemoryFeatureLayerLoadEnc.Columns.Add(new FeatureSourceColumn("Polygon"));
            //AVCSInMemoryFeatureLayerLoadEnc.ZoomLevelSet.ZoomLevel04.CustomStyles.Add(customDefaultValueStyle_Valid);
            //AVCSInMemoryFeatureLayerLoadEnc.Close();
            LoadMap();
            AVLayerOverlayLoadEnc.WrappingMode = WrappingMode.WrapDateline;
            AVLayerOverlayLoadEnc.WrappingExtent = new RectangleShape(-180, 90, 180, -90);
            AVLayerOverlayLoadEnc.TileType = TileType.SingleTile;

            AVInMemoryFeatureLayerLoadEnc.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(0, 0, 0, 0), GeoColor.FromArgb(500, 255, 0, 0), 1, LineDashStyle.Solid);
            AVInMemoryFeatureLayerLoadEnc.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            AVInMemoryFeatureLayerLoadEnc.WrappingMode = WrappingMode.WrapDateline;
            AVInMemoryFeatureLayerLoadEnc.WrappingExtent = new RectangleShape(-180, 90, 180, -90);/*mapControl.Overlays["Diff0Overlay"].GetBoundingBox();*/
            TextStyle textStyle = new TextStyle("Name", new GeoFont("Arial", 12), new GeoSolidBrush(GeoColor.StandardColors.HotPink));
            AVLayerOverlayLoadEnc.Layers.Clear();

            AVInMemoryFeatureLayerLoadEnc.Open();

            AVInMemoryFeatureLayerLoadEnc.EditTools.BeginTransaction();
            //AVInMemoryFeatureLayerLoadEnc.InternalFeatures.RemoveAll();
            foreach (string file in Directory.GetFiles(appDataPath, "*.txt", SearchOption.AllDirectories))
            {
                string wkt = File.ReadAllText(file);
                CustomFeature wrapPolygonFeature = new CustomFeature(wkt);
                wrapPolygonFeature.WrapGeomotry();
                // map.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(wrapPolygonFeature);
                AVInMemoryFeatureLayerLoadEnc.EditTools.Add(wrapPolygonFeature);
            }
            AVInMemoryFeatureLayerLoadEnc.EditTools.CommitTransaction();
            AVInMemoryFeatureLayerLoadEnc.Close();

            AVLayerOverlayLoadEnc.Layers.Add("AVInMemoryFeatureLayerLoadEnc", AVInMemoryFeatureLayerLoadEnc);
            //if (map.Overlays.Contains("AVCSLayerOverlayLoadEnc"))
            //    map.Overlays.Remove("AVCSLayerOverlayLoadEnc");
            map.Overlays.Add("AVLayerOverlayLoadEnc", AVLayerOverlayLoadEnc);

            layerOverlay = new LayerOverlay();
            layerOverlay.TileType = TileType.SingleTile;
            layerOverlay.WrappingMode = WrappingMode.WrapDateline;
            layerOverlay.WrappingExtent = new RectangleShape(-180, 90, 180, -90);
            highlightLayer = new InMemoryFeatureLayer();
            highlightLayer.WrappingMode = WrappingMode.WrapDateline;
            highlightLayer.WrappingExtent = new RectangleShape(-180, 90, 180, -90);
            highlightLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoBrushes.Red, 2));
            highlightLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(0, 0, 0, 0), GeoColor.FromArgb(500, 255, 0, 0), 1, LineDashStyle.Solid);
            highlightLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(new GeoFont(), 0, GeoBrushes.Red);
            highlightLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            layerOverlay.Layers.Add(highlightLayer);
            map.Overlays.Add(layerOverlay);

            map.Refresh();
        }
        private void LoadMap()
        {
            InMemoryFeatureLayer inMemoryFeatureDiff0Layer = new InMemoryFeatureLayer();
            // string rootPath = System.IO.Path.GetDirectoryName(System.Diagnostics.Process.GetCurrentProcess().MainModule.FileName);

            //if (File.Exists(rootPath + @"\Data\demo\world.shp"))
            //{
            //    ShapeFileFeatureLayer worldlayer = new ShapeFileFeatureLayer(rootPath + @"\Data\demo\world.shp");
            //    worldlayer.ZoomLevelSet.ZoomLevel10.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            //    //worldlayer.ZoomLevelSet.ZoomLevel10.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(500, 240, 237, 230), GeoColor.FromArgb(500, 185, 115, 176));
            //    // worldlayer.ZoomLevelSet.ZoomLevel10.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(500, 255, 255, 255), GeoColor.FromArgb(500, 180, 180, 180));
            //    //  worldlayer.ZoomLevelSet.ZoomLevel10.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(500, 188, 187, 185), GeoColor.FromArgb(500, 255, 255, 255));
            //    // worldlayer.ZoomLevelSet.ZoomLevel10.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(500, 255, 255, 255), GeoColor.FromArgb(500, 0, 0, 0));
            //    worldlayer.ZoomLevelSet.ZoomLevel10.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(500, 239, 237, 234), GeoColor.FromArgb(500, 0, 0, 0));
            //    worldlayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(AreaStyles.CreateSimpleAreaStyle
            //    (GeoColor.FromArgb(500, 240, 237, 230), GeoColor.FromArgb(500, 185, 115, 176), 1));

            //    LayerOverlay staticlayer = new LayerOverlay();
            //    staticlayer.Layers.Add(worldlayer);
            //    staticlayer.WrappingMode = WrappingMode.WrapDateline;
            //    map.Overlays.Add(staticlayer);
            //}

            {
                var shapeFilePath = @"..\..\shapefile\ne_50m_admin_0_countries.shp";
                if (File.Exists(shapeFilePath))
                {
                    inMemoryFeatureDiff0Layer = new InMemoryFeatureLayer();

                    inMemoryFeatureDiff0Layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level09;
                    //inMemoryFeatureDiff0Layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(500, 240, 237, 230), GeoColor.FromArgb(500, 185, 115, 176));
                    inMemoryFeatureDiff0Layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(500, 255, 255, 255), GeoColor.FromArgb(500, 180, 180, 180));
                    // inMemoryFeatureDiff0Layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(500, 255, 255, 255), GeoColor.FromArgb(500, 0, 0, 0));
                    //inMemoryFeatureDiff0Layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(500, 239, 237, 234), GeoColor.FromArgb(500, 180, 180, 180));

                    AddfeaturesToLayer(inMemoryFeatureDiff0Layer, GetAllFeaturesFromShape(shapeFilePath));

                }
            }
            AddOverlayToMap(inMemoryFeatureDiff0Layer, "Diff0Layer", "Diff0Overlay");
            //   AddOverlayToMap(inMemoryFeatureDiff1Layer, "Diff1Layer", "Diff1Overlay");


            //            if (File.Exists(rootPath + @"\Data\demo\paticens.shp"))
            //            {
            //                inMemoryFeatureDiff1Layer = new InMemoryFeatureLayer();
            //                // ShapeFileFeatureLayer Name = new ShapeFileFeatureLayer(rootPath + @"\Data\demo\parte.shp");
            //                // Name.ZoomLevelSet.ZoomLevel04.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //#pragma warning disable CS0618 // Type or member is obsolete
            //                inMemoryFeatureDiff1Layer.ZoomLevelSet.ZoomLevel04.DefaultPointStyle = PointStyles.City6;
            //                //Name.ZoomLevelSet.ZoomLevel04.DefaultTextStyle = TextStyles.City6("CITY_NAME");
            //                //Name.ZoomLevelSet.ZoomLevel04.DefaultTextStyle.BestPlacement = true;
            //                inMemoryFeatureDiff1Layer.ZoomLevelSet.ZoomLevel04.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            //                AddfeaturesToLayer(inMemoryFeatureDiff1Layer, GetAllFeaturesFromShape(rootPath + @"\Data\demo\paticens.shp"));

            //#pragma warning restore CS0618 // Type or member is obsolete
            //                //   LayerOverlay staticlayer1 = new LayerOverlay();
            //                //staticlayer1.Layers.Add(Name);
            //                //staticlayer1.WrappingMode = WrappingMode.WrapDateline;
            //                //orderMap.Overlays.Add(staticlayer1);
            //            }
            //            AddOverlayToMap(inMemoryFeatureDiff1Layer, "Diff2Layer", "Diff2Overlay");


            //            if (File.Exists(rootPath + @"\Data\demo\Countries02.shp"))
            //            {

            //                ShapeFileFeatureLayer Name = new ShapeFileFeatureLayer(rootPath + @"\Data\demo\Countries02.shp");
            //                Name.ZoomLevelSet.ZoomLevel05.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level10;
            //                Name.ZoomLevelSet.ZoomLevel05.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(0, 255, 255, 255), GeoColor.FromArgb(0, 255, 255, 255));
            //#pragma warning disable CS0618 // Type or member is obsolete
            //                Name.ZoomLevelSet.ZoomLevel05.DefaultPointStyle = PointStyles.City6;
            //                //Name.ZoomLevelSet.ZoomLevel04.DefaultTextStyle = TextStyles.City6("CNTRY_NAME");
            //                Name.ZoomLevelSet.ZoomLevel05.DefaultTextStyle = TextStyles.CreateSimpleTextStyle("CNTRY_NAME", "fira sans light", 11, DrawingFontStyles.Regular, GeoColor.StandardColors.Black);
            //                Name.ZoomLevelSet.ZoomLevel05.DefaultTextStyle.BestPlacement = true;
            //                Name.ZoomLevelSet.ZoomLevel05.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            //#pragma warning restore CS0618 // Type or member is obsolete
            //                LayerOverlay staticlayer1 = new LayerOverlay();
            //                staticlayer1.Layers.Add(Name);
            //                staticlayer1.WrappingMode = WrappingMode.WrapDateline;
            //                map.Overlays.Add(staticlayer1);

            //            }

            //  Utility.TriggerLoadingCompleted();

        }


        private void AddOverlayToMap(InMemoryFeatureLayer featureLayer, string layerName, string overlayName)
        {
            if (featureLayer != null)
            {
                LayerOverlay layerOverlay = new LayerOverlay();
                layerOverlay.WrappingMode = WrappingMode.WrapDateline;
                layerOverlay.WrappingExtent = new RectangleShape(-180, 90, 180, -90);
                layerOverlay.Layers.Add(layerName, featureLayer);
                map.Overlays.Add(overlayName, layerOverlay);
            }
        }

        private Collection<Feature> GetAllFeaturesFromShape(string shapeFilePath)
        {


            // ShapeFileFeatureSource.BuildIndexFile(@"E:\Projects\Phase-2\CornesWPF\bin\Debug\Data\demo\paticens.shp", @"E:\Projects\Phase-2\CornesWPF\bin\Debug\Data\demo\paticens.idx", BuildIndexMode.Rebuild);
            ShapeFileFeatureSource shapeFileFeatureSource = new ShapeFileFeatureSource(shapeFilePath);
            shapeFileFeatureSource.Open();
            var errors = shapeFileFeatureSource.Validate();
            Collection<Feature> features = shapeFileFeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            shapeFileFeatureSource.Close();
            return features;
        }

        private void AddfeaturesToLayer(InMemoryFeatureLayer layer, Collection<Feature> features)
        {
            layer.Open();
            layer.InternalFeatures.Clear();
            foreach (var item in features)
            {
                layer.InternalFeatures.Add(new Feature((item.MakeValid().GetWellKnownText())));
            }

            layer.Close();
        }

        private void MapClick(object sender, MapClickWpfMapEventArgs e)
        {
            if (selectedFeatures.Count > 0 && worldLocation.X == e.WorldLocation.X && worldLocation.Y == e.WorldLocation.Y)
            {
                int index = selectedFeatures.IndexOf(highlightLayer.InternalFeatures[0]);
                index = index < selectedFeatures.Count - 1 ? index + 1 : 0;
                highlightLayer.InternalFeatures.Clear();
                highlightLayer.InternalFeatures.Add(selectedFeatures[index]);
            }
            else
            {
                selectedFeatures.Clear();
                highlightLayer.InternalFeatures.Clear();
                foreach (var feature in map.TrackOverlay.TrackShapeLayer.InternalFeatures)
                {
                    CustomFeature tempFeature = new CustomFeature(e.WorldLocation);
                    tempFeature.WrapGeomotry();
                    if (feature.Contains(tempFeature))
                        selectedFeatures.Add(feature);
                }

                if (selectedFeatures.Count > 0)
                    highlightLayer.InternalFeatures.Add(selectedFeatures[0]);

                worldLocation = (PointShape)e.WorldLocation.CloneDeep();
            }

            map.Refresh(layerOverlay);
        }

        private void button_Click(object sender, RoutedEventArgs e)
        {
            Button button = sender as Button;
            if (button != null)
            {
                switch (button.Name)
                {
                    case "btnTrackNormal":
                        map.TrackOverlay.TrackMode = TrackMode.None;
                        break;

                    case "btnTrackPoint":
                        map.TrackOverlay.TrackMode = TrackMode.Point;
                        break;

                    case "btnTrackLine":
                        map.TrackOverlay.TrackMode = TrackMode.Line;
                        break;

                    case "btnTrackRectangle":
                        map.TrackOverlay.TrackMode = TrackMode.Rectangle;
                        break;

                    case "btnTrackSquare":
                        map.TrackOverlay.TrackMode = TrackMode.Square;
                        break;

                    case "btnTrackPolygon":
                        map.TrackOverlay.TrackMode = TrackMode.Polygon;
                        break;

                    case "btnTrackCircle":
                        map.TrackOverlay.TrackMode = TrackMode.Circle;
                        break;

                    case "btnTrackEllipse":
                        map.TrackOverlay.TrackMode = TrackMode.Ellipse;
                        break;

                    case "btnTrackEdit":
                        map.TrackOverlay.TrackMode = TrackMode.None;
                        foreach (Feature feature in map.TrackOverlay.TrackShapeLayer.InternalFeatures)
                        {
                            map.EditOverlay.EditShapesLayer.InternalFeatures.Add(feature);
                        }

                        map.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear();
                        map.EditOverlay.CalculateAllControlPoints();
                        map.Refresh(new Overlay[] { map.EditOverlay, map.TrackOverlay });
                        break;

                    case "btnTrackSave":
                        foreach (Feature feature in map.EditOverlay.EditShapesLayer.InternalFeatures)
                            map.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(feature);

                        map.EditOverlay.EditShapesLayer.InternalFeatures.Clear();
                        map.EditOverlay.CalculateAllControlPoints();
                        map.Refresh(new Overlay[] { map.EditOverlay, map.TrackOverlay });

                        string tempFolder = "Temp";
                        if (!Directory.Exists(tempFolder)) Directory.CreateDirectory(tempFolder);
                        foreach (string file in Directory.GetFiles(tempFolder)) File.Delete(file);

                        foreach (Feature feature in map.TrackOverlay.TrackShapeLayer.InternalFeatures)
                        {
                            string tempFileName = Path.Combine(tempFolder, feature.Id + ".txt");
                            CustomFeature tempFeature = new CustomFeature(feature);
                            tempFeature.UnwrapGeomotry();
                            File.WriteAllText(tempFileName, tempFeature.GetWellKnownText());
                        }
                        break;

                    case "btnTrackDelete":
                        // should use TrackMode.EditShape select a shape, and then call DeleteTrackShape();
                        int lastIndex = map.EditOverlay.EditShapesLayer.InternalFeatures.Count - 1;

                        if (lastIndex >= 0)
                        {
                            map.EditOverlay.EditShapesLayer.InternalFeatures.RemoveAt(lastIndex);
                            map.EditOverlay.CalculateAllControlPoints();
                        }

                        map.Refresh(map.EditOverlay);
                        break;

                    default:
                        map.TrackOverlay.TrackMode = TrackMode.None;
                        break;
                }
            }
        }
    }
}
